"""
Verify all dependencies for NLU are requried
        # 1. Verfiy PYSPARK INSTALLED
        # 2. Verify SPARK-NLP INSTALLED
        # 3. Verify JAVA8 IS DEFAULT
        # FAILURE :
        # IF ST IMPORTABLE, TRY WRITE WARN MESSAGE TO STREAMLIT FROM NLU.LOAD()
        # IF ST NOT IMPORTABLE. JUST PRINT ERR MESSAGE
        # 4. ELSE LINK TO INSTALL NOTES https://nlu.johnsnowlabs.com/docs/en/install  and SLACK https://join.slack.com/t/spark-nlp/shared_invite/zt-lutct9gm-kuUazcyFKhuGY3_0AMkxqA



"""

# TODO
def verify_pyspark_installed(): pass

def verify_spark_nlp_installed(): pass

def verify_java8_installed_and_default(correct=''): pass

def verify_python_version(correct=''): pass